@@ -208,9 +208,9 @@ gst_alsasrc_class_init (GstAlsaSrcClass * klass)
   gstbasesrc_class = (GstBaseSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasrc_finalize);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_alsasrc_get_property);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_alsasrc_set_property);
+  gobject_class->finalize = gst_alsasrc_finalize;
+  gobject_class->get_property = gst_alsasrc_get_property;
+  gobject_class->set_property = gst_alsasrc_set_property;
 
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_alsasrc_getcaps);
 
@@ -810,7 +810,7 @@ gst_alsasrc_read (GstAudioSrc * asrc, gpointer data, guint length)
   }
   GST_ALSA_SRC_UNLOCK (asrc);
 
-  return length - cptr;
+  return length - (cptr * alsa->bytes_per_sample);
 
 read_error:
   {
