@@ -250,6 +250,7 @@ gst_tag_parse_extended_comment (const gchar * ext_comment, gchar ** key,
  *
  * Convenience function to read a string with unknown character encoding. If
  * the string is already in UTF-8 encoding, it will be returned right away.
+ * If not it tries to detect byte-order-mark for UTF-16/32 cases and use that.
  * Otherwise, the environment will be searched for a number of environment
  * variables (whose names are specified in the NULL-terminated string array
  * @env_vars) containing a list of character encodings to try/use. If none
@@ -288,6 +289,52 @@ gst_tag_freeform_string_to_utf8 (const gchar * data, gint size,
     goto beach;
   }
 
+  /* check for and use byte-order-mark for UTF-16/32 cases */
+  if (size >= 2) {
+    gchar *c = NULL;
+    gint prefix, ssize;
+
+    if (size >= 4) {
+      prefix = 4;
+      ssize = GST_ROUND_DOWN_4 (size - 4);
+      switch (GST_READ_UINT32_BE (data)) {
+        case 0x0000FEFF:
+          c = "UTF-32BE";
+          break;
+        case 0xFFFE0000:
+          c = "UTF-32LE";
+          break;
+        default:
+          break;
+      }
+    }
+    if (!c) {
+      prefix = 2;
+      ssize = GST_ROUND_DOWN_2 (size - 2);
+      switch (GST_READ_UINT16_BE (data)) {
+        case 0xFEFF:
+          c = "UTF-16BE";
+          break;
+        case 0xFFFE:
+          c = "UTF-16LE";
+          break;
+        default:
+          break;
+      }
+    }
+    if (c) {
+      GST_LOG ("Trying to convert freeform string to UTF-8 from '%s'", c);
+      if ((utf8 =
+              g_convert (data + prefix, ssize, "UTF-8", c, &bytes_read, NULL,
+                  NULL))) {
+        if (bytes_read == ssize)
+          goto beach;
+        g_free (utf8);
+        utf8 = NULL;
+      }
+    }
+  }
+
   while (env_vars && *env_vars != NULL) {
     const gchar *env = NULL;
 
@@ -410,7 +457,6 @@ gst_tag_image_data_to_image_buffer (const guint8 * image_data,
 
   memcpy (GST_BUFFER_DATA (image), image_data, image_data_len);
   GST_BUFFER_DATA (image)[image_data_len] = '\0';
-  GST_BUFFER_SIZE (image) = image_data_len;
 
   /* Find GStreamer media type, can't trust declared type */
   caps = gst_type_find_helper_for_buffer (NULL, image, NULL);
@@ -434,7 +480,7 @@ gst_tag_image_data_to_image_buffer (const guint8 * image_data,
    * to keep the original size of the image
    */
   if (!g_str_equal (name, "text/uri-list"))
-    GST_BUFFER_SIZE (image) = GST_BUFFER_SIZE (image) - 1;
+    GST_BUFFER_SIZE (image) = image_data_len;
 
   if (image_type != GST_TAG_IMAGE_TYPE_NONE) {
     GST_LOG ("Setting image type: %d", image_type);
